package uk.co.martinpearman.b4a.liblame;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.intervigil.lame.Decoder;
import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.ShortName;

//	@ActivityObject
@Events(values = { "DecodeComplete(Success As Boolean)" })
@ShortName("liblameDecoder")
public class DecoderWrapper extends AbsObjectWrapper<Decoder> {
	//	class members
	private static String mEventName;
	private static boolean mIsDecoding = false;
	private static int mTaskId = 0;

	/**
	 * Start the decoding process.
	 *The event DecodeComplete(Success As Boolean) will be raised on completion.
	 */
	public void Decode(final BA pBA) {
		if (mIsDecoding == false && getObject()!=null) {
			Runnable runnable = new Runnable() {
				@Override
				public void run() {
					mIsDecoding = true;
					boolean success = false;
					try {
						getObject().decode();
						success = true;
					} catch (IOException e) {
						e.printStackTrace();
					}
					getObject().cleanup();
					setObject(null);
					mIsDecoding = false;
					pBA.raiseEventFromDifferentThread(DecoderWrapper.this, null, mTaskId++, mEventName + "_decodecomplete", true, new Object[] { success });
				}
			};
			BA.submitRunnable(runnable, null, mTaskId++);
		} else {
			pBA.raiseEvent(DecoderWrapper.this, mEventName + "_decodecomplete", new Object[] { false });
		}
	}
	
	/**
	 * Initialize the Decoder with an event name.
	 */
	public void Initialize(String EventName) {
		mEventName = EventName.toLowerCase(BA.cul);
		setObject(null);
	}

	@Override
	public boolean IsInitialized() {
		return getObjectOrNull()==null?false:true;
	}

	/**
	 * Set the source mp3 and destination wav file names.
	 *Returns True on success, False on error. 
	 */
	public boolean SetFiles(String InputFilename, String OutputFilename) {
		boolean success = false;
		try {
			if(getObjectOrNull()!=null){
				getObject().cleanup();
				setObject(null);
			}
			File inputFile = new File(InputFilename);
			File outputFile = new File(OutputFilename);
			Decoder decoder = new Decoder(inputFile, outputFile);
			decoder.initialize();
			success = true;
			setObject(decoder);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return success;
	}
}
