package uk.co.martinpearman.b4a.liblame;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import net.sourceforge.lame.Lame;

import com.intervigil.lame.Encoder;

import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

//	@ActivityObject
@Author("Martin Pearman")
@Events(values = { "EncodeComplete(Success As Boolean)" })
@ShortName("liblameEncoder")
@Version(1.01f)
public class EncoderWrapper extends AbsObjectWrapper<Encoder> {
	//	class members
	public static final int LAME_PRESET_DEFAULT = 0;
	public static final int LAME_PRESET_MEDIUM = 1;
	public static final int LAME_PRESET_STANDARD = 2;
	public static final int LAME_PRESET_EXTREME = 3;

	private static String mEventName;
	private static boolean mIsEncoding = false;
	private static int mTaskId = 0;

	/**
	 * Start the encoding process.
	 *The event EncodeComplete(Success As Boolean) will be raised on completion.
	 */
	public void Encode(final BA pBA) {
		if (mIsEncoding == false && getObject()!=null) {
			Runnable runnable = new Runnable() {
				@Override
				public void run() {
					mIsEncoding = true;
					boolean success = false;
					try {
						getObject().encode();
						success = true;
					} catch (IOException e) {
						e.printStackTrace();
					}
					getObject().cleanup();
					setObject(null);
					mIsEncoding = false;
					pBA.raiseEventFromDifferentThread(EncoderWrapper.this, null, mTaskId++, mEventName + "_encodecomplete", true, new Object[] { success });
				}
			};
			BA.submitRunnable(runnable, null, mTaskId++);
		} else {
			pBA.raiseEvent(EncoderWrapper.this, mEventName + "_encodecomplete", new Object[] { false });
		}
	}

	
	/**
	 * Initialize the Encoder with an event name.
	 */
	public void Initialize(String EventName) {
		mEventName = EventName.toLowerCase(BA.cul);
		setObject(null);
	}

	@Override
	public boolean IsInitialized() {
		return getObjectOrNull()==null?false:true;
	}

	/**
	 * Set the source wav and destination mp3 file names.
	 *The Encoder will default to Quality of Encoder.LAME_PRESET_DEFAULT.
	 *Returns True on success, False on error. 
	 */
	public boolean SetFiles(String InputFilename, String OutputFilename) {
		boolean success = false;
		try {
			if(getObjectOrNull()!=null){
				getObject().cleanup();
				setObject(null);
			}
			File inputFile = new File(InputFilename);
			File outputFile = new File(OutputFilename);
			Encoder encoder = new Encoder(inputFile, outputFile);
			encoder.initialize();
			encoder.setPreset(Lame.LAME_PRESET_DEFAULT);
			success = true;
			setObject(encoder);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return success;
	}

	/**
	 * Set the encoding quality.
	 *Choose from:
	 *Encoder.LAME_PRESET_DEFAULT
	 *Encoder.LAME_PRESET_MEDIUM
	 *Encoder.LAME_PRESET_STANDARD
	 *Encoder.LAME_PRESET_EXTREME
	 */
	public void SetQuality(int Quality) {
		getObject().setPreset(Quality);
	}
}
